/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tprsctrl.c
 *
 * Purpose: This is the target process black box resource control program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tprsctrl.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: tprsctrl.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>

#include "tp.h"
#include "ir_oid.h"
#include "tpconst.h"
#include "irerror.h"
#include "tpbbox.h"

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_resource_control_request
 *
 * Purpose: This function fills the resource control request PDU.
 *
 * Input Arguments:
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_ptr - filled and created PDU.
 *              response_required - TRUE  - resource control response is required
 *                                  FALSE - resource control response is not
 *                                          required
 *
 * External Variables:
 *
 * Function used:
 *              ASN_MakeOid()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_resource_request(PDU_ptr, response_required, reference_id_length, reference_id)
    APDU *PDU_ptr;
    int *response_required;
    AsnInt reference_id_length;
    char *reference_id;
{

    char resource_report_str[200];

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_resource_request\n");
#endif

    PDU_ptr->a.resourceControlReq = (struct ResourceControlReq *) malloc(sizeof(struct ResourceControlReq));
    memset(PDU_ptr->a.resourceControlReq, NULL, sizeof(struct ResourceControlReq));

    PDU_ptr->choiceId = APDU_RESOURCECONTROLREQ;

    PDU_ptr->a.resourceControlReq->referenceId.octetLen = reference_id_length;
    PDU_ptr->a.resourceControlReq->referenceId.octs = (char *) malloc(reference_id_length);
    memcpy(PDU_ptr->a.resourceControlReq->referenceId.octs, reference_id, reference_id_length);

    PDU_ptr->a.resourceControlReq->suspendedFlag = (AsnBool *) malloc(sizeof(AsnBool));
    *PDU_ptr->a.resourceControlReq->suspendedFlag = 0;

    /* Fill the field resourceReport */
    PDU_ptr->a.resourceControlReq->resourceReport = (EXTERNAL *) malloc(sizeof(EXTERNAL));

    ASN_MakeOid((char *) OID_CLASS_RESOURCE_REPORT_FORMAT, &PDU_ptr->a.resourceControlReq->resourceReport->direct_reference);

    PDU_ptr->a.resourceControlReq->resourceReport->indirect_reference = (AsnInt *) malloc(sizeof(AsnInt));
    *PDU_ptr->a.resourceControlReq->resourceReport->indirect_reference = 0;

    PDU_ptr->a.resourceControlReq->resourceReport->data_value_descriptor.octetLen = 0;
    PDU_ptr->a.resourceControlReq->resourceReport->data_value_descriptor.octs = NULL;

    PDU_ptr->a.resourceControlReq->resourceReport->encoding = (struct EXTERNALChoice *) malloc(sizeof(struct EXTERNALChoice));
    PDU_ptr->a.resourceControlReq->resourceReport->encoding->choiceId = EXTERNALCHOICE_OCTET_ALIGNED;
    PDU_ptr->a.resourceControlReq->resourceReport->encoding->a.octet_aligned = (AsnOcts *) malloc(sizeof(AsnOcts));

    sprintf(resource_report_str, "Resource Report (TTVVCC)");
    PDU_ptr->a.resourceControlReq->resourceReport->encoding->a.octet_aligned->octetLen = strlen(resource_report_str);
    PDU_ptr->a.resourceControlReq->resourceReport->encoding->a.octet_aligned->octs = (char *) malloc(strlen(resource_report_str));
    memcpy(PDU_ptr->a.resourceControlReq->resourceReport->encoding->a.octet_aligned->octs, resource_report_str, PDU_ptr->a.resourceControlReq->resourceReport->encoding->a.octet_aligned->octetLen);
    /* End of fill the field resourceReport */

    PDU_ptr->a.resourceControlReq->responseRequired = FALSE;
    *response_required = PDU_ptr->a.resourceControlReq->responseRequired;

    PDU_ptr->a.resourceControlReq->triggeredRequestFlag = (AsnBool *) malloc(sizeof(AsnBool));
    *PDU_ptr->a.resourceControlReq->triggeredRequestFlag = 0;

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_create_resource_control_request
 *
 * Purpose: This function creates the resource control request PDU and encodes
 *          it.
 *
 * Input Arguments:
 *          data_ptr - temporary data buffer during encoding process.
 *
 * Return values:
 *
 * Output Arguments:
 *          outputBuf - encoded APDU from the created PDU.
 *          APDU_length - the length of the encoded APDU.
 *          response_required - TRUE  - resource control response is required
 *                              FALSE - resource control response is not
 *                                      required
 *
 * External Variables:
 *
 * Function used:
 *          tp_fill_resource control_request()
 *          InitNibbleMem()
 *          SBufInstallData()
 *          SBufResetInWriteRvsMode()
 *          BEncAPDU()
 *          ir_fwrite()
 *          FreeAccessControlReq()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_resource_request(outputBuf, data_ptr, APDU_length, response_required, reference_id_length, reference_id)
    SBuf *outputBuf;
    char **data_ptr;
    int *APDU_length;
    int *response_required;
    AsnInt reference_id_length;
    char *reference_id;
{
    APDU PDU_ptr;
    int i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_resource_request()\n");
#endif

    tp_fill_resource_request(&PDU_ptr, response_required, reference_id_length, reference_id);

    InitNibbleMem(MAXMESSAGESIZE, MAXMESSAGESIZE);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:Resource Request APDU Dump from tprsctrl\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    FreeResourceControlReq(PDU_ptr.a.resourceControlReq);

    return (SUCCESS);
}


/*****************************************************************************
 *
 * Function: tp_resource_control_control
 *
 * Purpose: This function creates an resource control request PDU, encodes it
 *          and then send to IR kernel. Secondly, it tries to receive the
 *          resource control response from IR kernel if the flag
 *          response_required is set to TRUE.
 *
 * Input Arguments:
 *              socketFd - socket Fd to send/receive the resource control APDU
 *                         to/from.
 *              data_ptr - temporary data buffer used by the ASN functions.
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tp_create_resource_control_request()
 *              tpa_send_APDU()
 *              display_error()
 *              tpa_service_event()
 *              tp_valid_user()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/
tp_resource_control(socketFd, data_ptr, reference_id_length, reference_id)
    int socketFd;
    char **data_ptr;
    AsnInt reference_id_length;
    char *reference_id;
{
    long APDU_length;
    SBuf APDU_ptr;
    APDU PDU;
    int response_required;

    if (!RESOURCE_CONTROL_REQUIRED)
        return (SUCCESS);

    /* send an resource control request to IR kernel */
    tp_create_resource_request(&APDU_ptr, data_ptr, &APDU_length, &response_required, reference_id_length, reference_id);

    if (tpa_send_APDU(socketFd, &APDU_ptr, APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }

    /* receive the resource control response from IR Kernel */
    if (response_required)
    {

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX: resource control response required\n");
#endif

        if (tpa_service_event(socketFd, &PDU) == SUCCESS)
        {
            return (SUCCESS);
        }
        else
            return (FAILED);
    }
    else
    {

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX: resource control response not required\n");
#endif
    }

    return (SUCCESS);
}
